/*
 * (C) Copyright IBM Corp. 2004
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This
 * file and program are licensed under a BSD style license.  See
 * the Copying file included with the OpenHPI distribution for
 * full licensing terms.
 *
 * Authors:
 *   Konrad Rzeszutek <konradr@us.ibm.com>
 * Note: this file originally auto-generated by mib2c using
 *       : mib2c.array-user.conf,v 5.18 2003/11/07 17:03:52 rstory Exp $
 *
 * $Id: saHpiSensorReadingMaxTable.c,v 1.2 2004/02/25 21:16:14 konradrzeszutek Exp $
 *
 *
 */
#include <SaHpi.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#include <net-snmp/library/snmp_assert.h>

#include <saHpiSensorReadingMaxTable.h>


#include <hpiSubagent.h>


static netsnmp_handler_registration *my_handler = NULL;
static netsnmp_table_array_callbacks cb;

oid saHpiSensorReadingMaxTable_oid[] =
  { saHpiSensorReadingMaxTable_TABLE_OID };
size_t saHpiSensorReadingMaxTable_oid_len =
OID_LENGTH (saHpiSensorReadingMaxTable_oid);

static int
modify_saHpiSensorReadingMaxTable_row (SaHpiDomainIdT domain_id,
					   SaHpiResourceIdT resource_id,
					   SaHpiSensorNumT sensor_num,
					   SaHpiEventCategoryT
					   sensor_category,
					   SaHpiSensorReadingT * reading,
					   saHpiSensorReadingMaxTable_context
					   * ctx);

int
delete_ReadingMax_row (SaHpiDomainIdT domain_id,
			   SaHpiResourceIdT resource_id,
			   SaHpiSensorNumT sensor_num)
{

  saHpiSensorReadingMaxTable_context *ctx;
  oid index_oid[SENSOR_READING_INDEX_NR];
  netsnmp_index sensor_reading_index;
  int rc = AGENT_ERR_NOT_FOUND;

  DEBUGMSGTL ((AGENT, "delete_ReadingMax_row (%d, %d, %d). Entry.\n",
	       domain_id, resource_id, sensor_num));

  // Look at the MIB to find out what the indexs are
  index_oid[0] = domain_id;
  index_oid[1] = resource_id;
  index_oid[2] = sensor_num;
  // Possible more indexs?
  sensor_reading_index.oids = (oid *) & index_oid;
  sensor_reading_index.len = SENSOR_READING_INDEX_NR;

  ctx = CONTAINER_FIND (cb.container, &sensor_reading_index);

  if (ctx)
    {
      CONTAINER_REMOVE (cb.container, ctx);
      saHpiSensorReadingMaxTable_delete_row (ctx);
      rc = AGENT_ERR_NOERROR;
    }
  DEBUGMSGTL ((AGENT, "delete_ReadingMax_row. Exit (rc: %d).\n", rc));
  return rc;
}


int
populate_ReadingMax (SaHpiDomainIdT domain_id,
			 SaHpiResourceIdT resource_id,
			 SaHpiSensorNumT sensor_id,
			 SaHpiEventCategoryT sensor_category,
			 SaHpiSensorReadingT * reading)
{


  int rc = AGENT_ERR_NOERROR;

  oid index_oid[SENSOR_READING_INDEX_NR];
  netsnmp_index sensor_reading_index;
  saHpiSensorReadingMaxTable_context *ctx;


  DEBUGMSGT ((AGENT, "\t--- populate_ReadingMax, Entry,\n"));

  if (reading)
    {

      /*
         Construct index 
       */
      sensor_reading_index.len = SENSOR_READING_INDEX_NR;
      index_oid[0] = domain_id;
      index_oid[1] = resource_id;
      index_oid[2] = sensor_id;
      sensor_reading_index.oids = (oid *) & index_oid;

      /*
       * Find the row.
       */
      ctx = NULL;
      ctx = CONTAINER_FIND (cb.container, &sensor_reading_index);

      if (!ctx)
	{
	  /* 
	     New entry. Create it.
	   */
	  ctx =
	    saHpiSensorReadingMaxTable_create_row (&sensor_reading_index);
	}
      if (!ctx)
	{
	  snmp_log (LOG_ERR,
		    "Not enough memory for a sensor ReadingMax row!");
	  return AGENT_ERR_INTERNAL_ERROR;
	}

      if (modify_saHpiSensorReadingMaxTable_row (domain_id,
						     resource_id,
						     sensor_id,
						     sensor_category,
						     reading,
						     ctx) == AGENT_NEW_ENTRY)
	{
	  /*
	   * Add new entry.
	   */
	  CONTAINER_INSERT (cb.container, ctx);
	}
      rc = AGENT_ERR_NOERROR;

    }
  else
    rc = AGENT_ERR_OPERATION;
  DEBUGMSGT ((AGENT, "\t--- populate_ReadingMax, Exit (rc: %d),\n", rc));

  return rc;
}


int
modify_saHpiSensorReadingMaxTable_row (SaHpiDomainIdT domain_id,
					   SaHpiResourceIdT resource_id,
					   SaHpiSensorNumT sensor_num,
					   SaHpiEventCategoryT
					   sensor_category,
					   SaHpiSensorReadingT * reading,
					   saHpiSensorReadingMaxTable_context
					   * ctx)
{

  long hash = 0;
  unsigned int update_entry = MIB_FALSE;
  //  char format[SENSOR_READING_INTER_MAX];

  DEBUGMSGTL ((AGENT, "Modify saHpiSensorReadingMaxTable_ctx: Entry.\n"));

  if (ctx)
    {
      hash = calculate_hash_value (reading, sizeof (SaHpiSensorReadingT));

      DEBUGMSGTL ((AGENT, " Hash value: %d, in ctx: %d\n", hash, ctx->hash));

      if (ctx->hash != 0)
	{
	  /* Only do the check if the hash value is something else than zero.
	   * 'zero' value is only for newly created records, and in some
	   * rare instances when the hash has rolled to zero - in which
	   * case we will just consider the worst-case scenario and update
	   * the record and not trust the hash value.
	   */
	  if (hash == ctx->hash)
	    {
	      /* The same data. No need to change. */
	      return AGENT_ENTRY_EXIST;
	    }
	  if ((ctx->domain_id == domain_id) &&
	      (ctx->resource_id == resource_id) &&
	      (ctx->sensor_id == sensor_num))
	    {
	      update_entry = MIB_TRUE;
	      DEBUGMSGTL ((AGENT,
			   "Updating ReadingMaxTable row [%d, %d, %d]\n",
			   domain_id, resource_id, sensor_num));
	    }
	}

      if (hash == 0)		/* Might happend - roll-over */
	hash = 1;		/* Need this - we consider hash
				 * values of '0' uninitialized */
      ctx->hash = hash;

      ctx->resource_id = resource_id;
      ctx->domain_id = domain_id;
      ctx->sensor_id = sensor_num;

      build_reading_strings (reading,
			     sensor_category,
			     &ctx->saHpiSensorReadingMaxValuesPresent,
			     &ctx->saHpiSensorReadingMaxRaw,
			     ctx->saHpiSensorReadingMaxInterpreted,
			     &ctx->saHpiSensorReadingMaxInterpreted_len,
			     SENSOR_READING_INTER_MAX,
			     &ctx->saHpiSensorReadingMaxStatus,
			     ctx->saHpiSensorReadingMaxEventStatus,
			     &ctx->saHpiSensorReadingMaxEventStatus_len,
			     SENSOR_READING_EVENT_MAX);

      /* END */
      DEBUGMSGTL ((AGENT, "Modify saHpiSensorReadingMaxTable_ctx: Exit"));
      if (update_entry == MIB_TRUE)
	return AGENT_ENTRY_EXIST;
      return AGENT_NEW_ENTRY;

    }

  DEBUGMSGTL ((AGENT,
	       "Modify saHpiSensorReadingMaxTable_ctx: Exit (NULL DATA)"));

  return AGENT_ERR_NULL_DATA;
}

/************************************************************
 * the *_row_copy routine
 */
static int
  saHpiSensorReadingMaxTable_row_copy
  (saHpiSensorReadingMaxTable_context * dst,
   saHpiSensorReadingMaxTable_context * src)
{
  if (!dst || !src)
    return 1;

  /*
   * copy index, if provided
   */
  if (dst->index.oids)
    free (dst->index.oids);
  if (snmp_clone_mem ((void *) &dst->index.oids, src->index.oids,
		      src->index.len * sizeof (oid)))
    {
      dst->index.oids = NULL;
      return 1;
    }
  dst->index.len = src->index.len;


  /*
   * copy components into the context structure
   */
  dst->saHpiSensorReadingMaxValuesPresent =
    src->saHpiSensorReadingMaxValuesPresent;

  dst->saHpiSensorReadingMaxRaw = src->saHpiSensorReadingMaxRaw;

  memcpy (dst->saHpiSensorReadingMaxInterpreted,
	  src->saHpiSensorReadingMaxInterpreted,
	  src->saHpiSensorReadingMaxInterpreted_len);
  dst->saHpiSensorReadingMaxInterpreted_len =
    src->saHpiSensorReadingMaxInterpreted_len;

  dst->saHpiSensorReadingMaxStatus = src->saHpiSensorReadingMaxStatus;

  memcpy (dst->saHpiSensorReadingMaxEventStatus,
	  src->saHpiSensorReadingMaxEventStatus,
	  src->saHpiSensorReadingMaxEventStatus_len);
  dst->saHpiSensorReadingMaxEventStatus_len =
    src->saHpiSensorReadingMaxEventStatus_len;
 dst->resource_id = src->resource_id;
    dst->domain_id = src->domain_id;
    dst->sensor_id = src->sensor_id;
    dst->hash = src->hash;
  return 0;
}



/**
 * the *_extract_index routine
 *
 * This routine is called when a set request is received for an index
 * that was not found in the table container. Here, we parse the oid
 * in the the individual index components and copy those indexes to the
 * context. Then we make sure the indexes for the new row are valid.
 */
int
  saHpiSensorReadingMaxTable_extract_index
  (saHpiSensorReadingMaxTable_context * ctx, netsnmp_index * hdr)
{
  /*
   * temporary local storage for extracting oid index
   *
   * extract index uses varbinds (netsnmp_variable_list) to parse
   * the index OID into the individual components for each index part.
   */
  netsnmp_variable_list var_saHpiDomainID;
  netsnmp_variable_list var_saHpiResourceID;
  netsnmp_variable_list var_saHpiSensorIndex;
  int err;

  /*
   * copy index, if provided
   */
  if (hdr)
    {
      netsnmp_assert (ctx->index.oids == NULL);
      if (snmp_clone_mem ((void *) &ctx->index.oids, hdr->oids,
			  hdr->len * sizeof (oid)))
	{
	  return -1;
	}
      ctx->index.len = hdr->len;
    }

  /*
   * initialize variable that will hold each component of the index.
   * If there are multiple indexes for the table, the variable_lists
   * need to be linked together, in order.
   */
  memset (&var_saHpiDomainID, 0x00, sizeof (var_saHpiDomainID));
  var_saHpiDomainID.type = ASN_UNSIGNED;	/* type hint for parse_oid_indexes */

  var_saHpiDomainID.next_variable = &var_saHpiResourceID;

  memset (&var_saHpiResourceID, 0x00, sizeof (var_saHpiResourceID));
  var_saHpiResourceID.type = ASN_UNSIGNED;	/* type hint for parse_oid_indexes */

  var_saHpiResourceID.next_variable = &var_saHpiSensorIndex;

  memset (&var_saHpiSensorIndex, 0x00, sizeof (var_saHpiSensorIndex));
  var_saHpiSensorIndex.type = ASN_UNSIGNED;	/* type hint for parse_oid_indexes */

  var_saHpiSensorIndex.next_variable = NULL;

  /*
   * parse the oid into the individual index components
   */
  err = parse_oid_indexes (hdr->oids, hdr->len, &var_saHpiDomainID);
  if (err == SNMP_ERR_NOERROR)
    {
      /*
       * copy index components into the context structure
       */
      /*
         IBM-KR: We don't copy them b/c they are not needed yet.
       */
	      /** skipping external index saHpiDomainID */

	      /** skipping external index saHpiResourceID */

	      /** skipping external index saHpiSensorIndex */
    }

  /*
   * parsing may have allocated memory. free it.
   */
  snmp_reset_var_buffers (&var_saHpiDomainID);

  return err;
}

/************************************************************
 * the *_can_delete routine is called to determine if a row
 * can be deleted.
 *
 * return 1 if the row can be deleted
 * return 0 if the row cannot be deleted
 */
int
  saHpiSensorReadingMaxTable_can_delete
  (saHpiSensorReadingMaxTable_context * undo_ctx,
   saHpiSensorReadingMaxTable_context * row_ctx,
   netsnmp_request_group * rg)
{

  return 1;
}

/************************************************************
 * the *_create_row routine is called by the table handler
 * to create a new row for a given index. If you need more
 * information (such as column values) to make a decision
 * on creating rows, you must create an initial row here
 * (to hold the column values), and you can examine the
 * situation in more detail in the *_set_reserve1 or later
 * states of set processing. Simple check for a NULL undo_ctx
 * in those states and do detailed creation checking there.
 *
 * returns a newly allocated saHpiSensorReadingMaxTable_context
 *   structure if the specified indexes are not illegal
 * returns NULL for errors or illegal index values.
 */
saHpiSensorReadingMaxTable_context *
saHpiSensorReadingMaxTable_create_row (netsnmp_index * hdr)
{
  saHpiSensorReadingMaxTable_context *ctx =
    SNMP_MALLOC_TYPEDEF (saHpiSensorReadingMaxTable_context);
  if (!ctx)
    return NULL;

  if (saHpiSensorReadingMaxTable_extract_index (ctx, hdr))
    {
      free (ctx->index.oids);
      free (ctx);
      return NULL;
    }

  return ctx;
}


/************************************************************
 * the *_duplicate row routine
 */
saHpiSensorReadingMaxTable_context
  *saHpiSensorReadingMaxTable_duplicate_row
  (saHpiSensorReadingMaxTable_context * row_ctx)
{
  saHpiSensorReadingMaxTable_context *dup;

  if (!row_ctx)
    return NULL;

  dup = SNMP_MALLOC_TYPEDEF (saHpiSensorReadingMaxTable_context);
  if (!dup)
    return NULL;

  if (saHpiSensorReadingMaxTable_row_copy (dup, row_ctx))
    {
      free (dup);
      dup = NULL;
    }

  return dup;
}

/************************************************************
 * the *_delete_row method is called to delete a row.
 */
netsnmp_index *saHpiSensorReadingMaxTable_delete_row
  (saHpiSensorReadingMaxTable_context * ctx)
{
  /*
   * netsnmp_mutex_destroy(ctx->lock); 
   */

  if (ctx->index.oids)
    free (ctx->index.oids);

  /*
   * release header
   */
  free (ctx);

  return NULL;
}


/************************************************************
 * RESERVE is used to check the syntax of all the variables
 * provided, that the values being set are sensible and consistent,
 * and to allocate any resources required for performing the SET.
 * After this stage, the expectation is that the set ought to
 * succeed, though this is not guaranteed. (In fact, with the UCD
 * agent, this is done in two passes - RESERVE1, and
 * RESERVE2, to allow for dependancies between variables).
 *
 * BEFORE calling this routine, the agent will call duplicate_row
 * to create a copy of the row (unless this is a new row; i.e.
 * row_created == 1).
 *
 * next state -> SET_RESERVE2 || SET_FREE
 */
void
saHpiSensorReadingMaxTable_set_reserve1 (netsnmp_request_group * rg)
{

}

void
saHpiSensorReadingMaxTable_set_reserve2 (netsnmp_request_group * rg)
{

}

/************************************************************
 * Assuming that the RESERVE phases were successful, the next
 * stage is indicated by the action value ACTION. This is used
 * to actually implement the set operation. However, this must
 * either be done into temporary (persistent) storage, or the
 * previous value stored similarly, in case any of the subsequent
 * ACTION calls fail.
 *
 * In your case, changes should be made to row_ctx. A copy of
 * the original row is in undo_ctx.
 */
void
saHpiSensorReadingMaxTable_set_action (netsnmp_request_group * rg)
{

}

/************************************************************
 * Only once the ACTION phase has completed successfully, can
 * the final COMMIT phase be run. This is used to complete any
 * writes that were done into temporary storage, and then release
 * any allocated resources. Note that all the code in this phase
 * should be "safe" code that cannot possibly fail (cue
 * hysterical laughter). The whole intent of the ACTION/COMMIT
 * division is that all of the fallible code should be done in
 * the ACTION phase, so that it can be backed out if necessary.
 *
 * BEFORE calling this routine, the agent will update the
 * container (inserting a row if row_created == 1, or removing
 * the row if row_deleted == 1).
 *
 * AFTER calling this routine, the agent will delete the
 * undo_info.
 */
void
saHpiSensorReadingMaxTable_set_commit (netsnmp_request_group * rg)
{

}

/************************************************************
 * If either of the RESERVE calls fail, the write routines
 * are called again with the FREE action, to release any resources
 * that have been allocated. The agent will then return a failure
 * response to the requesting application.
 *
 * AFTER calling this routine, the agent will delete undo_info.
 */
void
saHpiSensorReadingMaxTable_set_free (netsnmp_request_group * rg)
{

}

/************************************************************
 * If the ACTION phase does fail (for example due to an apparently
 * valid, but unacceptable value, or an unforeseen problem), then
 * the list of write routines are called again, with the UNDO
 * action. This requires the routine to reset the value that was
 * changed to its previous value (assuming it was actually changed),
 * and then to release any resources that had been allocated. As
 * with the FREE phase, the agent will then return an indication
 * of the error to the requesting application.
 *
 * BEFORE calling this routine, the agent will update the container
 * (remove any newly inserted row, re-insert any removed row).
 *
 * AFTER calling this routing, the agent will call row_copy
 * to restore the data in existing_row from the date in undo_info.
 * Then undo_info will be deleted (or existing row, if row_created
 * == 1).
 */
void
saHpiSensorReadingMaxTable_set_undo (netsnmp_request_group * rg)
{

}




/************************************************************
 *
 * Initialize the saHpiSensorReadingMaxTable table by defining its contents and how it's structured
 */
void
initialize_table_saHpiSensorReadingMaxTable (void)
{
  netsnmp_table_registration_info *table_info;

  if (my_handler)
    {
      snmp_log (LOG_ERR,
		"initialize_table_saHpiSensorReadingMaxTable_handler called again\n");
      return;
    }

  memset (&cb, 0x00, sizeof (cb));

    /** create the table structure itself */
  table_info = SNMP_MALLOC_TYPEDEF (netsnmp_table_registration_info);

  /*
   * if your table is read only, it's easiest to change the
   * HANDLER_CAN_RWRITE definition below to HANDLER_CAN_RONLY 
   */
  my_handler =
    netsnmp_create_handler_registration
    ("saHpiSensorReadingMaxTable",
     netsnmp_table_array_helper_handler,
     saHpiSensorReadingMaxTable_oid,
     saHpiSensorReadingMaxTable_oid_len, HANDLER_CAN_RONLY);

  if (!my_handler || !table_info)
    {
      snmp_log (LOG_ERR, "malloc failed in "
		"initialize_table_saHpiSensorReadingMaxTable_handler\n");
      return;	/** mallocs failed */
    }

    /***************************************************
     * Setting up the table's definition
     */

  /*
   * internal indexes
   */
	/** index: saHpiDomainID */
  netsnmp_table_helper_add_index (table_info, ASN_UNSIGNED);
	/** index: saHpiResourceID */
  netsnmp_table_helper_add_index (table_info, ASN_UNSIGNED);
	/** index: saHpiSensorIndex */
  netsnmp_table_helper_add_index (table_info, ASN_UNSIGNED);

  table_info->min_column = saHpiSensorReadingMaxTable_COL_MIN;
  table_info->max_column = saHpiSensorReadingMaxTable_COL_MAX;

    /***************************************************
     * registering the table with the master agent
     */
  cb.get_value = saHpiSensorReadingMaxTable_get_value;
  cb.container =
    netsnmp_container_find ("saHpiSensorReadingMaxTable_primary:"
			    "saHpiSensorReadingMaxTable:"
			    "table_container");


  cb.create_row = (UserRowMethod *) saHpiSensorReadingMaxTable_create_row;

  cb.duplicate_row =
    (UserRowMethod *) saHpiSensorReadingMaxTable_duplicate_row;
  cb.delete_row = (UserRowMethod *) saHpiSensorReadingMaxTable_delete_row;
  cb.row_copy =
    (Netsnmp_User_Row_Operation *) saHpiSensorReadingMaxTable_row_copy;

  cb.can_delete =
    (Netsnmp_User_Row_Action *) saHpiSensorReadingMaxTable_can_delete;

  cb.set_reserve1 = saHpiSensorReadingMaxTable_set_reserve1;
  cb.set_reserve2 = saHpiSensorReadingMaxTable_set_reserve2;
  cb.set_action = saHpiSensorReadingMaxTable_set_action;
  cb.set_commit = saHpiSensorReadingMaxTable_set_commit;
  cb.set_free = saHpiSensorReadingMaxTable_set_free;
  cb.set_undo = saHpiSensorReadingMaxTable_set_undo;

  DEBUGMSGTL (("initialize_table_saHpiSensorReadingMaxTable",
	       "Registering table saHpiSensorReadingMaxTable "
	       "as a table array\n"));
  netsnmp_table_container_register (my_handler, table_info, &cb,
				    cb.container, 1);
}

/************************************************************
 * saHpiSensorReadingMaxTable_get_value
 *
 * This routine is called for get requests to copy the data
 * from the context to the varbind for the request. If the
 * context has been properly maintained, you don't need to
 * change in code in this fuction.
 */
int
saHpiSensorReadingMaxTable_get_value (netsnmp_request_info * request,
					  netsnmp_index * item,
					  netsnmp_table_request_info
					  * table_info)
{
  netsnmp_variable_list *var = request->requestvb;
  saHpiSensorReadingMaxTable_context *context =
    (saHpiSensorReadingMaxTable_context *) item;

  switch (table_info->colnum)
    {

    case COLUMN_SAHPISENSORREADINGMAXVALUESPRESENT:
	    /** INTEGER = ASN_INTEGER */
      snmp_set_var_typed_value (var, ASN_INTEGER,
				(char *) &context->
				saHpiSensorReadingMaxValuesPresent,
				sizeof (context->
					saHpiSensorReadingMaxValuesPresent));
      break;

    case COLUMN_SAHPISENSORREADINGMAXRAW:
	    /** UNSIGNED32 = ASN_UNSIGNED */
      snmp_set_var_typed_value (var, ASN_UNSIGNED,
				(char *) &context->
				saHpiSensorReadingMaxRaw,
				sizeof (context->
					saHpiSensorReadingMaxRaw));
      break;

    case COLUMN_SAHPISENSORREADINGMAXINTERPRETED:
	    /** OCTETSTR = ASN_OCTET_STR */
      snmp_set_var_typed_value (var, ASN_OCTET_STR,
				(char *) &context->
				saHpiSensorReadingMaxInterpreted,
				context->
				saHpiSensorReadingMaxInterpreted_len);
      break;

    case COLUMN_SAHPISENSORREADINGMAXSTATUS:
	    /** INTEGER = ASN_INTEGER */
      snmp_set_var_typed_value (var, ASN_INTEGER,
				(char *) &context->
				saHpiSensorReadingMaxStatus,
				sizeof (context->
					saHpiSensorReadingMaxStatus));
      break;

    case COLUMN_SAHPISENSORREADINGMAXEVENTSTATUS:
	    /** OCTETSTR = ASN_OCTET_STR */
      snmp_set_var_typed_value (var, ASN_OCTET_STR,
				(char *) &context->
				saHpiSensorReadingMaxEventStatus,
				context->
				saHpiSensorReadingMaxEventStatus_len);
      break;

    default:
	    /** We shouldn't get here */
      snmp_log (LOG_ERR, "unknown column in "
		"saHpiSensorReadingMaxTable_get_value\n");
      return SNMP_ERR_GENERR;
    }
  return SNMP_ERR_NOERROR;
}
